<?php

namespace Pinpoint\Conesso\Cron;

use Magento\Framework\App\Cache\Type\Config;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\Stdlib\DateTime\DateTime;
use Pinpoint\Conesso\Model\Api\Management\Process\ProcessFactory;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Pinpoint\Conesso\Model\Config\CartSettings;

class Cart
{
    /**
     * @var ProcessFactory
     */
    private ProcessFactory $processFactory;

    /**
     * @var WriterInterface
     */
    private WriterInterface $configWriter;

    /**
     * @var DateTime
     */
    private DateTime $dateTime;

    /**
     * @var TypeListInterface
     */
    private TypeListInterface $cacheTypeList;

    /**
     * @var CartSettings
     */
    private CartSettings $cartSettings;

    /**
     * @param ProcessFactory $processFactory
     * @param CartSettings $cartSettings
     * @param WriterInterface $configWriter
     * @param DateTime $dateTime
     * @param TypeListInterface $cacheTypeList
     */
    public function __construct(
        ProcessFactory    $processFactory,
        CartSettings      $cartSettings,
        WriterInterface   $configWriter,
        DateTime          $dateTime,
        TypeListInterface $cacheTypeList
    )
    {
        $this->processFactory = $processFactory;
        $this->cartSettings = $cartSettings;
        $this->configWriter = $configWriter;
        $this->dateTime = $dateTime;
        $this->cacheTypeList = $cacheTypeList;
    }

    /**
     * @return void
     */
    public function execute(): void
    {
        if ($this->cartSettings->isModuleEnabled() && $this->cartSettings->isCronEnabled()) {
            if ($this->processFactory->getProcess('cart')->synchronizeData()) {
                $this->configWriter->save(CartSettings::XML_PATH_LAST_CRON_RUN_TIMESTAMP, $this->dateTime->gmtDate());
                $this->cacheTypeList->cleanType(Config::TYPE_IDENTIFIER);
            }
        }
    }
}
