<?php
namespace Pinpoint\Conesso\Model\Api\Management\Log\Carts;

use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Stdlib\DateTime;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\Conesso\Api\Data\CartSyncInterface;
use Pinpoint\Conesso\Api\Data\CartSyncInterfaceFactory;
use Pinpoint\Conesso\Api\Repository\CartsSyncRepositoryInterface;

class LogCart implements LogCartInterface
{
    /**
     * @var CartSyncInterfaceFactory
     */
    private CartSyncInterfaceFactory $cartSyncInterfaceFactory;

    /**
     * @var CartsSyncRepositoryInterface
     */
    private CartsSyncRepositoryInterface $cartSyncRepository;

    /**
     * @var StoreManagerInterface
     */
    private StoreManagerInterface $storeManager;

    /**
     * @param CartsSyncRepositoryInterface $cartSyncRepository
     * @param CartSyncInterfaceFactory $cartSyncInterfaceFactory
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        CartsSyncRepositoryInterface $cartSyncRepository,
        CartSyncInterfaceFactory $cartSyncInterfaceFactory,
        StoreManagerInterface $storeManager
    ) {
        $this->cartSyncRepository = $cartSyncRepository;
        $this->cartSyncInterfaceFactory = $cartSyncInterfaceFactory;
        $this->storeManager = $storeManager;
    }

    /**
     * @param CartInterface $cart
     * @param $eventType
     * @return void
     * @throws NoSuchEntityException
     */
    public function logCart(CartInterface $cart, $eventType): void
    {
        $syncedCart = $this->cartSyncRepository->getById($cart->getId());
        if($syncedCart->getId()) {
            $this->updateCart($syncedCart);
        } else {
            $this->addCart($cart, $eventType);
        }
    }

    /**
     * Update existing cart
     * @param $syncedCart
     * @return void
     */
    private function updateCart($syncedCart): void
    {
        $syncedCart->setData(CartSyncInterface::IS_SYNCED, 0);
        $syncedCart->setData(CartSyncInterface::UPDATED_AT, (new \DateTime())->format(DateTime::DATETIME_PHP_FORMAT));
        $this->cartSyncRepository->save($syncedCart);
    }

    /**
     * Add new cart in `connesso_cart_sync`
     * @param $cart
     * @param $eventType
     * @return void
     * @throws NoSuchEntityException
     */
    private function addCart($cart, $eventType): void
    {
        $cartData = $this->cartSyncInterfaceFactory->create();
        $data = [
            CartSyncInterface::STORE_ID => (int) $this->storeManager->getStore()->getStoreId(),
            CartSyncInterface::CART_ID => (int) $cart->getId(),
            CartSyncInterface::EVENT_TYPE => $eventType,
            CartSyncInterface::IS_SYNCED => 0,
        ];
        $cartData->setData($data);
        $this->cartSyncRepository->save($cartData);
    }
}
