<?php
namespace Pinpoint\Conesso\Model\Api\Management\Log\Customer;

use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\Conesso\Api\Data\ContactSyncInterface;
use Pinpoint\Conesso\Api\Data\ContactSyncInterfaceFactory;
use Pinpoint\Conesso\Api\Repository\ContactsRepositoryInterface;

class LogCustomer implements LogCustomerInterface
{
    /**
     * @var ContactSyncInterfaceFactory
     */
    private ContactSyncInterfaceFactory $contactSyncFactory;

    /**
     * @var ContactsRepositoryInterface
     */
    private ContactsRepositoryInterface $syncContactRepository;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @param ContactsRepositoryInterface $syncContactRepository
     * @param ContactSyncInterfaceFactory $contactSyncFactory
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ContactsRepositoryInterface $syncContactRepository,
        ContactSyncInterfaceFactory $contactSyncFactory,
        StoreManagerInterface       $storeManager
    ) {
        $this->syncContactRepository = $syncContactRepository;
        $this->contactSyncFactory = $contactSyncFactory;
        $this->storeManager = $storeManager;
    }

    /**
     * @param CustomerInterface $customer
     * @param $eventType
     * @return void
     * @throws NoSuchEntityException
     */
    public function logCustomer(CustomerInterface $customer, $eventType): void
    {
        $contactData = $this->contactSyncFactory->create();
        $data = [
            ContactSyncInterface::STORE_ID => (int) $this->storeManager->getStore()->getStoreId(),
            ContactSyncInterface::CUSTOMER_ID => (int) $customer->getId(),
            ContactSyncInterface::CUSTOMER_EMAIL => $customer->getEmail(),
            ContactSyncInterface::EVENT_TYPE => $eventType,
            ContactSyncInterface::IS_SYNCED => 0
        ];
        $contactData->setData($data);
        $this->syncContactRepository->save($contactData);
    }
}
