<?php
namespace Pinpoint\Conesso\Model\Api\Management\Log\Orders;

use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\Conesso\Api\Data\OrderSyncInterface;
use Pinpoint\Conesso\Api\Data\OrderSyncInterfaceFactory;
use Pinpoint\Conesso\Api\Repository\OrdersSyncRepositoryInterface;

class LogOrder implements LogOrderInterface
{
    /**
     * @var OrderSyncInterfaceFactory
     */
    private OrderSyncInterfaceFactory $orderSyncFactory;

    /**
     * @var OrdersSyncRepositoryInterface
     */
    private OrdersSyncRepositoryInterface $ordersSyncRepository;

    /**
     * @var StoreManagerInterface
     */
    private StoreManagerInterface $storeManager;

    /**
     * @param OrdersSyncRepositoryInterface $ordersSyncRepository
     * @param OrderSyncInterfaceFactory $orderSyncFactory
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        OrdersSyncRepositoryInterface $ordersSyncRepository,
        OrderSyncInterfaceFactory $orderSyncFactory,
        StoreManagerInterface $storeManager
    ) {
        $this->ordersSyncRepository = $ordersSyncRepository;
        $this->orderSyncFactory = $orderSyncFactory;
        $this->storeManager = $storeManager;
    }

    /**
     * @param OrderInterface $order
     * @param $eventType
     * @return void
     * @throws NoSuchEntityException
     */
    public function logOrder(OrderInterface $order, $eventType): void
    {
        $orderData = $this->orderSyncFactory->create();
        $data = [
            OrderSyncInterface::STORE_ID => (int) $this->storeManager->getStore()->getStoreId(),
            OrderSyncInterface::ORDER_ID => (int) $order->getEntityId(),
            OrderSyncInterface::INCREMENT_ID => $order->getIncrementId(),
            OrderSyncInterface::EVENT_TYPE => $eventType,
            OrderSyncInterface::IS_SYNCED => 0,
        ];
        $orderData->setData($data);
        $this->ordersSyncRepository->save($orderData);
    }
}
