<?php
namespace Pinpoint\Conesso\Model\Api\Management\Log\Products;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\Conesso\Api\Data\ProductSyncInterface;
use Pinpoint\Conesso\Api\Data\ProductSyncInterfaceFactory;
use Pinpoint\Conesso\Api\Repository\ProductsSyncRepositoryInterface;

class LogProduct implements LogProductInterface
{
    /**
     * @var ProductSyncInterfaceFactory
     */
    private ProductSyncInterfaceFactory $productSyncFactory;

    /**
     * @var ProductsSyncRepositoryInterface
     */
    private ProductsSyncRepositoryInterface $syncProductRepository;

    /**
     * @var StoreManagerInterface
     */
    private StoreManagerInterface $storeManager;

    /**
     * @param ProductsSyncRepositoryInterface $syncProductRepository
     * @param ProductSyncInterfaceFactory $productSyncFactory
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ProductsSyncRepositoryInterface $syncProductRepository,
        ProductSyncInterfaceFactory $productSyncFactory,
        StoreManagerInterface $storeManager
    ) {
        $this->syncProductRepository = $syncProductRepository;
        $this->productSyncFactory = $productSyncFactory;
        $this->storeManager = $storeManager;
    }

    /**
     * @param ProductInterface $product
     * @param $eventType
     * @return void
     * @throws NoSuchEntityException
     */
    public function logProduct(ProductInterface $product, $eventType): void
    {
        $productData = $this->productSyncFactory->create();
        $data = [
            ProductSyncInterface::STORE_ID => (int) $this->storeManager->getStore()->getStoreId(),
            ProductSyncInterface::PRODUCT_ID =>  $product->getId(),
            ProductSyncInterface::PRODUCT_SKU =>  $product->getSku(),
            ProductSyncInterface::EVENT_TYPE => $eventType,
            ProductSyncInterface::IS_SYNCED => 0,

        ];
        $productData->setData($data);
        $this->syncProductRepository->save($productData);
    }
}
