<?php
namespace Pinpoint\Conesso\Model\Api\Management\Transfer\Request;

use Exception;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory;
use Magento\Framework\DataObject;
use Pinpoint\Conesso\Api\Data\OrderSyncInterface;
use Pinpoint\Conesso\Api\Repository\OrdersSyncRepositoryInterface;
use Pinpoint\Conesso\Model\Api\Access\Auth;
use Pinpoint\Conesso\Model\Config\OrderSettings;

class OrderRequest extends AbstractRequest
{
    /**
     * @var OrdersSyncRepositoryInterface
     */
    protected OrdersSyncRepositoryInterface $ordersSyncRepository;

    /**
     * @var CollectionFactory
     */
    protected CollectionFactory $orderCollectionFactory;

    /**
     * @var OrderSettings
     */
    private OrderSettings $orderSettings;

    /**
     * @param Auth $authorization
     * @param OrdersSyncRepositoryInterface $ordersSyncRepository
     * @param CollectionFactory $orderCollectionFactory
     * @param OrderSettings $orderSettings
     */
    public function __construct(
        Auth $authorization,
        OrdersSyncRepositoryInterface $ordersSyncRepository,
        CollectionFactory $orderCollectionFactory,
        OrderSettings $orderSettings
    ) {
        $this->ordersSyncRepository = $ordersSyncRepository;
        $this->orderCollectionFactory = $orderCollectionFactory;
        $this->orderSettings = $orderSettings;
        parent::__construct($authorization);
    }

    /**
     * @param mixed $order
     * @return array
     */
    public function buildData(mixed $order): array
    {
        //TODO: for order
        $data = [
            'contactId' => 37,
            'customerEmail' => 'ergys.sahatcija@idhl.co.uk',
            'customerFirstname' => null,
            'customerLastname' => null,
            'customerGender' => '',
            'customerTelephone' => '+12341234123',
            'customerMobile' => '',
            'customerDob' => null,
            'customerIsGuest' => 1,
            'customerNotes' => '',
            'billingFirstname' => 'Ergys',
            'billingLastname' => 'S',
            'billingAddress1' => '123 Test Road',
            'billingAddress2' => 'Testing Lane',
            'billingCity' => 'TestVille',
            'billingCounty' => 'TestingShire',
            'billingPostcode' => 'TS1 1ST',
            'billingCountry' => 'GB',
            'shippingFirstname' => 'Ergys',
            'shippingLastname' => 'S',
            'shippingAddress1' => '123 Test Road',
            'shippingAddress2' => 'Testing Lane',
            'shippingCity' => 'TestVille',
            'shippingCounty' => 'TestingShire',
            'shippingCountry' => 'GB',
            'shippingPostcode' => 'TS1 1ST',
            'total' => 5000,
            'totalIncTax' => 6000,
            'isoCurrencyCode' => 'GBP',
            'numberOfProducts' => 1,
            'cartProducts' => [
                'productId' => 'api-24-WG081-blue',
                'productSku' => '24-WG081-blue',
                'productName' => 'Sprite Stasis Ball 55 cm',
                'productDescription' => 'Test Product Description',
                'taxPercent' => 20,
                'productPriceTax' => 2.50,
                'productOriginalPrice' => 30.00,
                'productPrice' => 20,
                'productPriceIncTax' => 22.50,
                'rowTotal' => 40,
                'rowTotalIncTax' => 45,
                'abandonedRowTotal' => 40,
                'abandonedRowTotalWithDiscount' => 40,
                'abandonedRowTotalDiscount' => 0,
                'quantity' => 1
            ]
        ];
        return $data;
    }

    /**
     * @param mixed $data
     * @return bool
     */
    public function sendRequest(mixed $data): bool
    {
        //TODO: for order
        foreach ($data['items'] as $item) {
            if ($item->getEventType() == OrderSyncInterface::EVENT_TYPE_SAVE) {
                $this->saveOrder($item);
            }
            if ($item->getEventType() == OrderSyncInterface::EVENT_TYPE_DELETE)
            {
                $this->deleteOrderInConesso($item);
            }
            $this->updatedSynchronizedData($item);
        }
        return true;
    }

    /**
     * @param $item
     * @return void
     */
    private function updatedSynchronizedData($item): void
    {
        $item->setIsSynced(1);
        $this->ordersSyncRepository->save($item);
    }

    /**
     * @param $item
     * @return void
     */
    private function saveOrder($item): void
    {
        //TODO: Load Order
        if ($order = $this->getOrderById($item->getOrderId())) {
            $parameters = $this->buildData($order);
            $this->getConnessoClient()->orders()->create($parameters);
//            $customerEmail = $order->getEmail();
//            if(!$this->existsOrderInConesso($customerEmail)) {
//                $this->getConnessoClient()->orders()->create($parameters);
//            }
        }
    }

    /**
     * @param $customerEmail
     * @return false|mixed
     */
    private function existsOrderInConesso($customerEmail): mixed
    {
        try {
            $contacts = $this->getConnessoClient()->contacts()->list()->data;

            foreach ($contacts as $contact) {
                if ($contact['email'] == $customerEmail) {
                    return $contact['id'];
                }
            }
            return false;
        } catch (TypeError|Exception $e) {
            return false;
        }
    }

    /**
     * @param $customerId
     * @return false|DataObject
     */
    private function getOrderById($customerId): DataObject|bool
    {
        try {
            return $this->orderCollectionFactory->create()->addFieldToFilter('entity_id', $customerId)->getFirstItem();
        } catch (Exception $e) {
            return  false;
        }
    }

    /**
     * @param $item
     * @return void
     */
    private function deleteOrderInConesso($item): void
    {
        $orderId = $this->existsOrderInConesso($item->getEmail());
        $this->getConnessoClient()->orders()->delete($orderId);
    }
}
